"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types_1 = require("tns-core-modules/utils/types");
var peer = {
    enabled: false,
    allowInvalidCertificates: false,
    validatesDomainName: true
};
var cache;
function setCache(options) {
    if (options) {
        cache = new okhttp3.Cache(new java.io.File(options.diskLocation), options.diskSize);
    }
    else {
        cache = null;
    }
    if (Client) {
        getClient(true);
    }
}
exports.setCache = setCache;
function clearCache() {
    if (cache) {
        cache.evictAll();
    }
}
exports.clearCache = clearCache;
var _timeout = 10;
var Callback = (function (_super) {
    __extends(Callback, _super);
    function Callback() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Callback.prototype.onStringResponse = function (content, statusCode, heads) {
        var headers = {};
        var i, len = heads.size();
        for (i = 0; i < len; i++) {
            var key = heads.name(i);
            headers[key] = heads.value(i);
        }
        this.resolve({ content: content, statusCode: statusCode, headers: headers });
    };
    Callback.prototype.onFailure = function (task, error) {
        this.reject(error);
    };
    return Callback;
}(com.nativescript.https.OkhttpCallback));
function enableSSLPinning(options) {
    if (!peer.host && !peer.certificate) {
        var certificate = void 0;
        var inputStream = void 0;
        try {
            var file = new java.io.File(options.certificate);
            inputStream = new java.io.FileInputStream(file);
            var x509Certificate = java.security.cert.CertificateFactory.getInstance('X509').generateCertificate(inputStream);
            peer.x509Certificate = x509Certificate;
            certificate = okhttp3.CertificatePinner.pin(x509Certificate);
            inputStream.close();
        }
        catch (error) {
            try {
                if (inputStream) {
                    inputStream.close();
                }
            }
            catch (e) {
            }
            console.error('nativescript-https > enableSSLPinning error', error);
            return;
        }
        peer.host = options.host;
        peer.commonName = options.commonName || options.host;
        peer.certificate = certificate;
        if (options.allowInvalidCertificates === true) {
            peer.allowInvalidCertificates = true;
        }
        if (options.validatesDomainName === false) {
            peer.validatesDomainName = false;
        }
    }
    peer.enabled = true;
    getClient(true);
    console.log('nativescript-https > Enabled SSL pinning');
}
exports.enableSSLPinning = enableSSLPinning;
function disableSSLPinning() {
    peer.enabled = false;
    getClient(true);
    console.log('nativescript-https > Disabled SSL pinning');
}
exports.disableSSLPinning = disableSSLPinning;
console.info('nativescript-https > Disabled SSL pinning by default');
var Client;
function getClient(reload, timeout) {
    if (reload === void 0) { reload = false; }
    if (timeout === void 0) { timeout = 10; }
    if (Client && reload === false && _timeout === timeout) {
        return Client;
    }
    _timeout = timeout;
    var client = new okhttp3.OkHttpClient.Builder();
    if (peer.enabled === true) {
        if (peer.host || peer.certificate) {
            var spec = okhttp3.ConnectionSpec.MODERN_TLS;
            client.connectionSpecs(java.util.Collections.singletonList(spec));
            var pinner = new okhttp3.CertificatePinner.Builder();
            pinner.add(peer.host, [peer.certificate]);
            client.certificatePinner(pinner.build());
            if (peer.allowInvalidCertificates === false) {
                try {
                    var x509Certificate = peer.x509Certificate;
                    var keyStore = java.security.KeyStore.getInstance(java.security.KeyStore.getDefaultType());
                    keyStore.load(null, null);
                    keyStore.setCertificateEntry('CA', x509Certificate);
                    var keyManagerFactory = javax.net.ssl.KeyManagerFactory.getInstance('X509');
                    keyManagerFactory.init(keyStore, null);
                    var keyManagers = keyManagerFactory.getKeyManagers();
                    var trustManagerFactory = javax.net.ssl.TrustManagerFactory.getInstance(javax.net.ssl.TrustManagerFactory.getDefaultAlgorithm());
                    trustManagerFactory.init(keyStore);
                    var sslContext = javax.net.ssl.SSLContext.getInstance('TLS');
                    sslContext.init(keyManagers, trustManagerFactory.getTrustManagers(), new java.security.SecureRandom());
                    client.sslSocketFactory(sslContext.getSocketFactory());
                }
                catch (error) {
                    console.error('nativescript-https > client.allowInvalidCertificates error', error);
                }
            }
            if (peer.validatesDomainName === true) {
                try {
                    client.hostnameVerifier(new javax.net.ssl.HostnameVerifier({
                        verify: function (hostname, session) {
                            var pp = session.getPeerPrincipal().getName();
                            var hv = javax.net.ssl.HttpsURLConnection.getDefaultHostnameVerifier();
                            if (peer.commonName && peer.commonName[0] === "*") {
                                return (hv.verify(peer.host, session) &&
                                    hostname.indexOf(peer.host) > -1 &&
                                    hostname.indexOf(session.getPeerHost()) > -1 &&
                                    pp.indexOf(peer.commonName) !== -1);
                            }
                            else {
                                return (hv.verify(peer.host, session) &&
                                    peer.host === hostname &&
                                    peer.host === session.getPeerHost() &&
                                    pp.indexOf(peer.host) !== -1);
                            }
                        },
                    }));
                }
                catch (error) {
                    console.error('nativescript-https > client.validatesDomainName error', error);
                }
            }
        }
        else {
            console.warn('nativescript-https > Undefined host or certificate. SSL pinning NOT working!!!');
        }
    }
    if (cache) {
        client.cache(cache);
    }
    if (timeout) {
        client
            .connectTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
            .writeTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS)
            .readTimeout(timeout, java.util.concurrent.TimeUnit.SECONDS);
    }
    Client = client.build();
    return Client;
}
function request(opts) {
    return new Promise(function (resolve, reject) {
        try {
            var client = getClient(false, opts.timeout);
            var request_1 = new okhttp3.Request.Builder();
            request_1.url(opts.url);
            if (opts.headers) {
                Object.keys(opts.headers).forEach(function (key) { return request_1.addHeader(key, opts.headers[key]); });
            }
            if (opts.cachePolicy) {
                var cacheControlBuilder = new okhttp3.CacheControl.Builder();
                switch (opts.cachePolicy) {
                    case "noCache":
                        cacheControlBuilder = cacheControlBuilder.noStore();
                        break;
                    case "onlyCache":
                        cacheControlBuilder = cacheControlBuilder.onlyIfCached();
                        break;
                    case "ignoreCache":
                        cacheControlBuilder = cacheControlBuilder.noCache();
                        break;
                }
                request_1.cacheControl(cacheControlBuilder.build());
            }
            var methods = {
                'GET': 'get',
                'HEAD': 'head',
                'DELETE': 'delete',
                'POST': 'post',
                'PUT': 'put',
                'PATCH': 'patch'
            };
            var type = void 0;
            if ((['GET', 'HEAD'].indexOf(opts.method) !== -1) || (opts.method === 'DELETE' && !types_1.isDefined(opts.body) && !types_1.isDefined(opts.content))) {
                request_1[methods[opts.method]]();
            }
            else {
                type = opts.headers && opts.headers['Content-Type'] ? opts.headers['Content-Type'] : 'application/json';
                var MEDIA_TYPE = okhttp3.MediaType.parse(type);
                var okHttpBody = void 0;
                if (type === 'multipart/form-data') {
                    var builder_1 = new okhttp3.MultipartBody.Builder();
                    builder_1.setType(MEDIA_TYPE);
                    Object.keys(opts.body).forEach(function (k) {
                        var param = opts.body[k];
                        if (param.fileName && param.contentType) {
                            var MEDIA_TYPE_1 = okhttp3.MediaType.parse(param.contentType);
                            builder_1.addFormDataPart(param.parameterName, param.fileName, okhttp3.RequestBody.create(MEDIA_TYPE_1, param.data));
                        }
                        else {
                            builder_1.addFormDataPart(param.parameterName, param.data);
                        }
                    });
                    okHttpBody = builder_1.build();
                }
                else {
                    var body = void 0;
                    if (opts.body) {
                        try {
                            body = JSON.stringify(opts.body);
                        }
                        catch (ignore) {
                        }
                    }
                    else if (opts.content) {
                        body = opts.content;
                    }
                    okHttpBody = okhttp3.RequestBody.create(okhttp3.MediaType.parse(type), body);
                }
                request_1[methods[opts.method]](okHttpBody);
            }
            if (opts.allowLargeResponse) {
                android.os.StrictMode.setThreadPolicy(android.os.StrictMode.ThreadPolicy.LAX);
            }
            var callback = new Callback();
            callback.resolve = resolve;
            callback.reject = reject;
            client.newCall(request_1.build()).enqueue(callback);
        }
        catch (error) {
            reject(error);
        }
    });
}
exports.request = request;
//# sourceMappingURL=https.android.js.map